/* dmatch.c
 * 
 * Copyright (C) 2002 Claudio Girardi  <in3otd@qsl.net>
 * 
 * This file was derived from the original Pascal code, written by
 * Stephen A. Maas. It was translated into ANSI C with the help of p2c,
 * version 1.21alpha, and further manually refined by Claudio Girardi.
 * 
 * Bugs were certainly introduced during the translation process, so
 * please don't bother Sthephen with bug reports but send them to
 * Claudio, thanks.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA, 02111-1307, USA
 */


/* Original file: DMATCH.PAS */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

/*
 * DMATCH is a dumb little program which investigates the matching
 * possibilities for a waveguide mixer.  It calculates the
 * resistance presented to the diode junction (R1 or R2) and the
 * concomitant backshort reactance (X1 or X2, respectively)
 * necessary to achieve a match.  All quantities are normalized to
 * the waveguide impedance (power/voltage definition), and if a
 * match is possible, two solutions are obtained.  DMATCH calculates
 * all possible match conditions between normalized inductive
 * reactances (L) .01 to 9.0.  If a match is impossible for some
 * value of L, it is ignored
 */


#ifndef M_PI
#define M_PI              3.141592654
#endif /* not M_PI */

#define FNSIZE 120

#ifndef true
# define true    1
# define false   0
#endif

#ifndef TRUE
# define TRUE    1
# define FALSE   0
#endif


static double c, zc, freq, cnorm, l, a, b, ck, d, r1, r2, x1, x2, e1, e2, f1, f2;
static short j, k;
static char yes_or_no;
static FILE *printout;
static unsigned char wantsaprint;
static char printout_NAME[FNSIZE];


static double pwroften_(short k)
{
  short j;
  double x = 1.0;

  for (j = 1; j <= abs(k); j++)
    x = 10.0 * x;
  if (k < 0)
    x = 1.0 / x;
  return x;
}


int main(int argc, char *argv[])
{
  printout = NULL;
  printf("Program DMATCH: diode matching analysis for waveguide mixers\n\n\n");
  printf("Enter frequency (GHz), waveguide Zc, and Cj (fF): ");
  scanf("%lg%lg%lg%*[^\n]", &freq, &zc, &c);
  getchar();
  cnorm = 2.0 * M_PI * freq * 1.0e-6 * c * zc;
  putchar('\n');
  yes_or_no = 'n';
  wantsaprint = false;
  printf("\nWant to write results to a file? y/n: ");
  scanf("%c", &yes_or_no);
  if (yes_or_no == 'y' || yes_or_no == 'Y') {
    strcpy(printout_NAME, "dmatch.txt");
    if (printout != NULL)
      printout = freopen(printout_NAME, "w", printout);
    else
      printout = fopen(printout_NAME, "w");
    if (printout == NULL) {
      fprintf(stderr, "error: %s: file not found\n", printout_NAME);
      exit(-1);
    }
    wantsaprint = true;
  }
  printf("\nNormalized capacitive susceptance:%10.6f\n", cnorm);
  if (wantsaprint) {
    fprintf(printout, "\nFrequency, GHz: %6.2f\n", freq);
    fprintf(printout, "Guide impedance, ohms: %6.2f\n", zc);
    fprintf(printout, "Capacitance, fF: %6.2f\n", c);
    fprintf(printout, "Normalized capacitive susceptance:%10.6f\n\n", cnorm);
  }
  printf("\n     XL/Zc     X1/Zc     R1/Zc     X2/Zc     R2/Zc\n");
  if (wantsaprint)
    fprintf(printout, "     XL/Zc     X1/Zc     R1/Zc     X2/Zc     R2/Zc\n");
  for (k = 1; k <= 3; k++) {
    for (j = 1; j <= 9; j++) {
      l = j * pwroften_(k) / 1000.0;
      if (fabs(l - 1 / cnorm) <= 1.0e-6)
	goto _L999;
      b = 1.0 - 2.0 * l * cnorm;
      a = l - l * l * cnorm - cnorm;
      if (fabs(a) <= 1.0e-50)
	goto _L999;
      ck = l - l * l * cnorm;
      d = b * b - 4.0 * a * ck;
      if (d <= 0.0)
	goto _L999;
      x1 = (sqrt(d) - b) / (2.0 * a);
      x2 = (-sqrt(d) - b) / (2.0 * a);
      e1 = l * cnorm + cnorm * x1 - 1.0;
      f1 = x1 - x1 * l * cnorm;
      if (fabs(e1 * e1 + f1 * f1) <= 1.0e-50)
	goto _L999;
      r1 = x1 * x1 / (e1 * e1 + f1 * f1);
      e2 = l * cnorm + cnorm * x2 - 1.0;
      f2 = x2 - x2 * l * cnorm;
      if (fabs(e2 * e2 + f2 * f2) <= 1.0e-50)
	goto _L999;
      r2 = x2 * x2 / (e2 * e2 + f2 * f2);
      printf("%11.5f%10.5f%10.5f%10.5f%10.5f\n", l, x1, r1, x2, r2);
      if (wantsaprint)
	fprintf(printout, "%11.5f%10.5f%10.5f%10.5f%10.5f\n", l, x1, r1, x2, r2);
    _L999:;
    }
  }
  printf("\n");
  if (printout != NULL) {
    fclose(printout);
    printf("File %s written\n", printout_NAME);
  }
  printout = NULL;

  exit(0);
}
